from unittest import TestCase


class TestPerfettoService(TestCase):
    def test_start(self):
        self.assertTrue(True)

    def test_stop(self):
        self.assertTrue(True)

    def test_export(self):
        self.assertTrue(True)

    def test_clean(self):
        self.assertTrue(True)
